require 'spec_helper'
require 'securerandom'
include ChtApitestMdr::MdrHelpers

module MdrFlexReportTest

  describe 'FlexReports', :async_apis do
    include_context 'request metadata'

    before(:all) do
      @report_name = "RSPEC ASYNC PREVIEW MONTHLY #{ENV['NAMESPACE']} - #{rand(999999999999999)}"
      @multi_part_api = MultiPartQueryApi.new
      @async_preview_ids = []
      @report_ids = []
      @dataset = 'AWS_CUR'
      start_time_day = Time.new - 11540000
      @from_month = (start_time_day).strftime("%Y-%m")
      @to_month = (start_time_day+13328000).strftime("%Y-%m")
      @from_month_daily_date = start_time_day.strftime("%Y-%m-%d")
      @to_month_daily_date = (start_time_day+7689600).strftime("%Y-%m-%d")
      @aggregated_sql = "SELECT timeInterval_Month AS Month, ELEMENT_AT(organization.tags, 'Account') AS organization_tags_Account, ELEMENT_AT(organization.tags, 'cht_owner') AS organization_tags_cht_owner ,perspective.accounts AS Perspective_accounts, organization.accountname AS Organization_accountname, lineItem_ProductCode AS ProductCode, lineItem_UsageType AS UsageType, lineItem_UsageAccountId AS UsageAccountId, costcategory_cht_tower AS CostCategoryChtTower, bill_CloudHealthGeneratedAccountId AS BillChGeneratedAccId, SUM(lineItem_UnblendedCost) AS SUM_LineItemUnblendedCost, SUM(edp_NetUnblendedCost) AS SUM_edpNetUnblendedCost, \"resourcetags_user:cht_tower\" AS Resourcetags_user_cht_tower, \"resourcetags_user:cht_owner\" AS Resourcetags_user_cht_owner, \"resourcetags_user:cht_function\" AS Resourcetags_user_cht_function, \"resourcetags_user:cht_env\" AS Resourcetags_user_cht_env FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_ProductCode, lineItem_UsageType, lineItem_UsageAccountId, costcategory_cht_tower, bill_CloudHealthGeneratedAccountId, perspective.accounts, organization.accountname, \"resourcetags_user:cht_tower\", \"resourcetags_user:cht_owner\", \"resourcetags_user:cht_function\", \"resourcetags_user:cht_env\", ELEMENT_AT(organization.tags, 'Account'), ELEMENT_AT(organization.tags, 'cht_owner') ORDER BY TimeInterval_Month DESC"
      @aggregated_sql ="SELECT timeInterval_Month AS Month, ELEMENT_AT(organization.tags, 'Account') AS organization_tags_Account, ELEMENT_AT(organization.tags, 'cht_owner') AS organization_tags_cht_owner ,perspective.accounts AS Perspective_accounts, organization.accountname AS Organization_accountname, lineItem_ProductCode AS ProductCode, lineItem_UsageType AS UsageType, lineItem_UsageAccountId AS UsageAccountId, costcategory_cht_tower AS CostCategoryChtTower, bill_CloudHealthGeneratedAccountId AS BillChGeneratedAccId, SUM(lineItem_UnblendedCost) AS SUM_LineItemUnblendedCost, SUM(LineItem_UsageAmount) AS SUM_lineItemUsageAmount, \"resourcetags_user:cht_tower\" AS Resourcetags_user_cht_tower, \"resourcetags_user:cht_owner\" AS Resourcetags_user_cht_owner, \"resourcetags_user:cht_function\" AS Resourcetags_user_cht_function, \"resourcetags_user:cht_env\" AS Resourcetags_user_cht_env FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_ProductCode, lineItem_UsageType, lineItem_UsageAccountId, costcategory_cht_tower, bill_CloudHealthGeneratedAccountId, perspective.accounts, organization.accountname, \"resourcetags_user:cht_tower\", \"resourcetags_user:cht_owner\", \"resourcetags_user:cht_function\", \"resourcetags_user:cht_env\", ELEMENT_AT(organization.tags, 'Account'), ELEMENT_AT(organization.tags, 'cht_owner') ORDER BY TimeInterval_Month DESC" if ENV['TEST_ENV'].eql? 'preprod'
      @default_x = "Month"
      @default_ys = ["SUM_LineItemUnblendedCost"]
      @default_z = "organization_tags_Account"
      @x = %w[Month organization_tags_Account organization_tags_cht_owner Perspective_accounts Organization_accountname ProductCode UsageType UsageAccountId CostCategoryChtTower BillChGeneratedAccId Resourcetags_user_cht_tower Resourcetags_user_cht_owner Resourcetags_user_cht_function Resourcetags_user_cht_env]
      @y = %w[SUM_LineItemUnblendedCost SUM_edpNetUnblendedCost]
      @y = %w[SUM_LineItemUnblendedCost SUM_lineItemUsageAmount] if ENV['TEST_ENV'].eql? 'preprod'
      @z = %w[Month organization_tags_Account organization_tags_cht_owner Perspective_accounts Organization_accountname ProductCode UsageType UsageAccountId CostCategoryChtTower BillChGeneratedAccId Resourcetags_user_cht_tower Resourcetags_user_cht_owner Resourcetags_user_cht_function Resourcetags_user_cht_env]
      @default_chart_type = 'column-stacked'
    end

    before(:each) do
      @headers = headers
    end

    it 'Create Report With Chart Config Selected(Column-Stacked) with MONTHLY Granularity', :create do
      wrap_request do
        variables = {
          'input': {
            'name': @report_name,
            'notification': {
              'sendUserEmail': false,
              'otherEmailIds': []
            },
            'chart': {
              'columns': [],
              'x': @x,
              'y': @y,
              'z': @z,
              'defaultX': @default_x,
              'defaultYs': @default_ys,
              'defaultZ': @default_z,
              'defaultChartType': @default_chart_type
            },
            'query': {
              'dataGranularity': 'MONTHLY',
              'limit': -1,
              'needBackLinkingForTags': true,
              'sqlStatement': @aggregated_sql,
              'timeRange': {
                'from': @from_month,
                'to': @to_month
              }
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createFlexReport,
                  service: service
                ),
                variables, @headers
        expect_success_graphql
        @report_ids << json_body[:data][:createFlexReport][:id]
        expect(%w[QUEUED RUNNING]).to include(json_body[:data][:createFlexReport][:result][:status])
        expect(json_body[:data][:createFlexReport][:query][:dataGranularity]).to eql 'MONTHLY'
      end
    end

    it 'Can Poll the Status of the Report Created', :status do
      poll_start_time = Time.now.to_i
      loop do
        @headers = regenerate_headers if Time.now.to_i > poll_start_time + 300
        wrap_request do
          variables = {}
          graphql url(service: service),
                  format_request(
                    :node,
                    service: service,
                    crn: @report_ids.first,
                    type: 'FlexReport',
                    fields: 'name archived result { contentType contents { preSignedUrl } message reportUpdatedOn status } chart { columns { columnUnit dataType filterable name namespace type usageAffinity } x y z defaultX defaultYs defaultZ defaultChartType } query { sqlStatement dataGranularity timeRange { from to } }'
                  ),
                  variables, @headers
          expect_success_graphql
          check(:node, service: service)
        end
        puts "Polling the Report with Report Name: #{json_body[:data][:node][:name]}, Status: #{json_body[:data][:node][:result][:status]}"
        # In Staging there won't be much load so checking if the report can be completed under 5 mins, Increase this to 10 mins if the PR Builds
        # FAil Constantly at this test case with Runtime Execption with below message
        raise("Error Occurred While Polling the Status of Report: Report #{json_body[:data][:node][:name]} is in #{json_body[:data][:node][:result][:status]} state for more than 5 mins") if Time.now.to_i > poll_start_time + 300000
        break unless %w[QUEUED RUNNING].include? json_body[:data][:node][:result][:status]
      end
      expect(json_body[:data][:node][:id]).to eql @report_ids.first
      expect(json_body[:data][:node][:name]).to eq  @report_name
      expect(json_body[:data][:node][:result][:status]).to eql 'COMPLETED'
      expect(json_body[:data][:node][:result][:status]).not_to eql 'FAILED'
      expect(json_body[:data][:node][:chart][:defaultX]).to eql @default_x
      expect(json_body[:data][:node][:chart][:defaultYs]).to eql @default_ys
      expect(json_body[:data][:node][:chart][:defaultZ]).to eql @default_z
      expect(json_body[:data][:node][:chart][:defaultChartType]).to eql @default_chart_type
    end

    it 'can query on top of the Report created with Chart Config Chunked set to True', chunked: true do
      @temp_table_sql = "SELECT \"#{@default_x}\", SUM(\"#{@default_ys.first}\") AS \"#{@default_ys.first}\", \"#{@default_z}\" FROM \"EX#REPORT=#{@report_ids.first}\" GROUP BY \"#{@default_x}\", \"#{@default_z}\""
      wrap_request do
        variables = {
          'chunkedResult': true,
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'sqlStatement': @temp_table_sql,
            'limit': 5000
          }
        }
        graphql url(service: service),
                format_request(
                  :asyncPreviewFlexReport,
                  service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:asyncPreviewFlexReport, service: service)
        expect(json_body[:data][:asyncPreviewFlexReport][:mode]).to eq 'OVERRIDE'
        expect(json_body[:data][:asyncPreviewFlexReport][:status]).to eq 'RUNNING'
        expect(json_body[:data][:asyncPreviewFlexReport][:asyncPreviewId]).not_to eq ''
        expect(json_body[:data][:asyncPreviewFlexReport][:message]).to eq ''
        expect(json_body[:data][:asyncPreviewFlexReport][:percentCompleted]).to eq 0
        expect(json_body[:data][:asyncPreviewFlexReport][:preview][:csv]).to eq ''
        @async_preview_ids << json_body[:data][:asyncPreviewFlexReport][:asyncPreviewId]
      end
    end

    it 'checks the aysncPreviewFlexReport api request response is COMPLETED using asyncPreviewFlexReportResponse api' do
      wrap_request do
        variables = {
          'statusInput': {
            'asyncPreviewId': @async_preview_ids.first
          }
        }
        graphql url(service: service),
                format_request(
                  :asyncPreviewFlexReportResponseStatus,
                  service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:asyncPreviewFlexReportResponseStatus, service: service)
        @async_preview_ids << json_body[:data][:asyncPreviewFlexReportResponseStatus][:asyncPreviewId]
      end
      async_preview_id = @async_preview_ids.last
      loop do
        break if async_preview_id.nil?
        @json_body = @multi_part_api.poll_async_preview_api_response_status(async_preview_id, service, @headers)
        break unless ['RUNNING', 'CHUNKED'].include? @json_body[:data][:asyncPreviewFlexReportResponseStatus][:status]
        async_preview_id = @json_body[:data][:asyncPreviewFlexReportResponseStatus][:asyncPreviewId]
      end
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:status]).to eq('COMPLETED')
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:mode]).to eq 'OVERRIDE'
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:asyncPreviewId]).to eq nil
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:message]).to eq ''
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:percentCompleted]).to eq 100
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:preview][:csv]).not_to eq ''
    end

    it 'can clean up the above created report using deleteFlexReport' do
      report_id = @report_ids.first
      wrap_request do
        variables = {
          'id': report_id
        }
        graphql url(service: service),
                format_request(
                  :delete,
                  service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:delete, service: service)
        expect(json_body[:data][:delete]).to eql true
      end
    end
  end
end
