require 'spec_helper'
require 'securerandom'
include ChtApitestMdr::MdrHelpers

module MdrFlexReportTest

  describe 'FlexReports' do
    include_context 'request metadata'

    before(:all) do
      # Report Id generated during test
      @report_name = "rspec-integration-automation-flex-report #{ENV['NAMESPACE']} - #{rand(999999999999999)}"
      @report_ids = []
      @dataset = 'AWS_CUR'
      start_time_day = Time.new - 11540000
      @from_month = (start_time_day).strftime("%Y-%m")
      @to_month = (start_time_day+13328000).strftime("%Y-%m")
      @from_month_daily_date = start_time_day.strftime("%Y-%m-%d")
      @to_month_daily_date = (start_time_day+7689600).strftime("%Y-%m-%d")
      @sql = "SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS ResourceTags_user_cht_owner FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\""
      @sql = "SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS SUM_LineItem_UnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS ResourceTags_user_cht_owner FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\""
    end

    before(:each) do
      @headers = headers
    end

    it "can FETCH METADATA for cloud AWS" do
      wrap_request do
        variables = {
            dataset: @dataset
        }
        graphql url(service: service),
                format_request(
                    :dataSourceMetadata,
                    service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:dataSourceMetadata, service: service)
        expect(json_body[:data][:dataSourceMetadata][:dataset]).to eql @dataset
      end
    end

    it 'Preview Report DAILY' do
      wrap_request do
        variables = {
            'queryInput': {
                'dataGranularity': 'DAILY',
                'needBackLinkingForTags': true,
                'sqlStatement': @sql,
                'timeRange': {
                  'from': @from_month_daily_date,
                  'to': @to_month_daily_date
                }
            }
        }
        graphql url(service: service),
                format_request(
                    :previewFlexReport,
                    service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it 'Preview Report MONTHLY' do
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': @sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it 'Create Report With DAILY Granularity', :create do
      wrap_request do
        variables = {
            'input': {
                'name': @report_name,
                'notification': {
                    'sendUserEmail': true,
                    'otherEmailIds': []
                        },
                'query': {
                    'dataGranularity': 'DAILY',
                    'limit': -1,
                    'needBackLinkingForTags': true,
                    'sqlStatement': @sql,
                    'timeRange': {
                        'from': @from_month_daily_date,
                        'to': @to_month_daily_date
                    }
                }
            }
        }
        graphql url(service: service),
                format_request(
                    :createFlexReport,
                    service: service
                ),
                variables, @headers
        expect_success_graphql
        @report_ids << json_body[:data][:createFlexReport][:id]
        expect(%w[QUEUED RUNNING]).to include(json_body[:data][:createFlexReport][:result][:status])
        expect(json_body[:data][:createFlexReport][:query][:dataGranularity]).to eql 'DAILY'
        # check(:createFlexReport, service: service)
      end
    end

    it "check the Created Report Status is NOT FAILED" do
      report_id = @report_ids.first
      wrap_request do
        variables = {}
        graphql url(service: service),
                format_request(
                    :node,
                    service: service,
                    crn: report_id,
                    type: 'FlexReport',
                    fields: 'name archived result { contentType contents { preSignedUrl } message reportUpdatedOn status } query { sqlStatement dataGranularity timeRange { from to } }'
                ),
                variables, @headers
        expect_success_graphql
        check(:node, service: service)
        expect(json_body[:data][:node][:id]).to eql report_id
        expect(json_body[:data][:node][:name]).to eq  @report_name
        expect(json_body[:data][:node][:result][:status]).not_to eql 'FAILED'
      end
    end

    it 'Update Existing Report datGranularity TO MONTHLY' do
      report_id = @report_ids.first
      wrap_request do
        variables = {
            'id': report_id,
            'input': {
                'name': @report_name,
                'notification': {
                    'sendUserEmail': true,
                    'otherEmailIds': []
                },
                'query': {
                    'dataGranularity': 'MONTHLY',
                    'limit': -1,
                    'needBackLinkingForTags': true,
                    'sqlStatement': @sql,
                    'timeRange': {
                        'from': @from_month,
                        'to': @to_month
                    }
                }
            }
        }
        graphql url(service: service),
                format_request(
                    :updateFlexReport,
                    service: service
                ),
                variables, @headers
        expect_success_graphql
        expect(json_body[:data][:updateFlexReport][:result][:status]).to eql('QUEUED')
        expect(json_body[:data][:updateFlexReport][:query][:dataGranularity]).to eql('MONTHLY')
        # check(:updateFlexReport, service: service)
      end
    end

    it "check the Updated Report Status is NOT FAILED" do
      report_id = @report_ids.first
      wrap_request do
        variables = {}
        graphql url(service: service),
                format_request(
                    :node,
                    service: service,
                    crn: report_id,
                    type: 'FlexReport',
                    fields: 'name archived result { status } query { sqlStatement dataGranularity timeRange { from to } }'
                ),
                variables, @headers
        expect_success_graphql
        check(:node, service: service)
        expect(json_body[:data][:node][:id]).to eql report_id
        expect(json_body[:data][:node][:name]).to eql  @report_name
        expect(json_body[:data][:node][:result][:status]).not_to eql 'FAILED'
      end
    end

    it "can DELETE an existing report" do
      report_id = @report_ids.first
      wrap_request do
        variables = {
            'id': report_id
        }
        graphql url(service: service),
                format_request(
                    :delete,
                    service: service
                ),
                variables, @headers
        expect_success_graphql
        check(:delete, service: service)
        expect(json_body[:data][:delete]).to eql true
      end
    end

    xit 'E:no auth' do
      wrap_request do
        variables = {
          'archived': false,
          'dataSource': @data_source
        }
        graphql url(service: service),
                format_request(
                  :flexReports,
                  service: service
                ),
                variables, apitest_headers
        expect_status(401)
        expect_error_graphql(
          status: 401,
          errors: [{ message: 'Invalid or missing authentication token' }]
        )
      end
    end
  end
end
