require 'spec_helper'

module MdrTest
  describe 'Mdr executeFlexReportQuery' do
    include_context 'request metadata'

    before(:all) do
      start_time_day = Time.new - 11540000
      @from_month = (start_time_day).strftime('%Y-%m')
      @to_month = (start_time_day+13328000).strftime('%Y-%m')
      @from_month_daily_date = start_time_day.strftime('%Y-%m-%d')
      @to_month_daily_date = (start_time_day+7689600).strftime('%Y-%m-%d')
      @failures = []
      @data = []
      @interactive_api = InteractiveApi.new
      @cost_column = 'edp_netUnblendedCost'
      @cost_column = 'LineItem_UnblendedCost' if ENV['TEST_ENV'].eql? 'preprod'
      @report_id = 'crn:2017:flexreports/b182da8b-6506-4fb1-b3e4-5683d6db0fa6'
      @report_id = 'crn:57534:flexreports/3d00ea37-22ce-48aa-a89a-fd1a1ebfe001' if ENV['TEST_ENV'].eql? 'preprod'
    end

    before(:each) do
      @headers = headers
    end

    it 'can executeFlexReportQuery and getFlexReportQueryResult for AWS CUR Dataset - MONTHLY GRANULARITY', :swami do

      query = "SELECT timeInterval_Month AS Month, SUM(#{@cost_column}) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC"

      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 33333, 1000, true, false, 'false' )
      expect(failures).to be_empty
    end

    it 'can executeFlexReportQuery and getFlexReportQueryResult for AWS CUR Dataset - DAILY GRANULARITY' do
      query = "SELECT timeInterval_Month AS Month, SUM(#{@cost_column}) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC"

      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'DAILY', @from_month_daily_date, @to_month_daily_date, query, 33333, 1000 , true, false, 'false')
      expect(failures).to be_empty
    end

    it 'can query on top of created report using report Id instead of dataset' do
      query = "SELECT LineItem_Operation, SUM_lineItem_UnblendedCost, LineItem_ProductCode, LineItem_LineItemType, LineItem_UsageType, SUM_lineItem_UsageAmount, Bill_PayerAccountId, Organization_accountname, Month, LineItem_UsageAccountId FROM \"EX#REPORT=#{@report_id}\""
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'DAILY', @from_month_daily_date, @to_month_daily_date, query, 10000, 1000 , false, false, 'false')
      expect(failures).to be_empty
    end

    it 'can query on dataset which is non time bounded - PRICING dataset and Not default dataset' do
      query = "select InstanceType instancetype, derived_region location, Tenancy tenancy, DiscountedPricing pricing from DX_PRICING_AWS_EC2 where termType = 'OnDemand' and EndDate = '9999-12-31' and PreInstalledSw = 'NA' and OperatingSystem = 'Linux' and derived_region = 'us-east-1'"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 1000, 100 , false, false, 'false')
      expect(failures).to be_empty
    end

    it 'can query on any dataset with limit < pageSize' do
      query = "SELECT LineItem_Operation, SUM_lineItem_UnblendedCost, LineItem_ProductCode, LineItem_LineItemType, LineItem_UsageType, SUM_lineItem_UsageAmount, Bill_PayerAccountId, Organization_accountname, Month, LineItem_UsageAccountId FROM \"EX#REPORT=#{@report_id}\""
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'DAILY', @from_month_daily_date, @to_month_daily_date, query, 100, 1000 , false, false, 'false')
      expect(failures).to be_empty
    end

    it 'can query on any dataset with limit == pageSize' do
      query = "SELECT LineItem_Operation, SUM_lineItem_UnblendedCost, LineItem_ProductCode, LineItem_LineItemType, LineItem_UsageType, SUM_lineItem_UsageAmount, Bill_PayerAccountId, Organization_accountname, Month, LineItem_UsageAccountId FROM \"EX#REPORT=#{@report_id}\""
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'DAILY', @from_month_daily_date, @to_month_daily_date, query, 100, 100 , false, false, 'false')
      expect(failures).to be_empty
    end

    it 'should fail and return error message when SQL is malformed' do
      query = "SELECT timeInterval_Month AS Month, SUM(#{@cost_column}) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC"
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 33333, 1000, true, false, 'true' )
      expect(error).to eq("Unable to parse the sql: 'SELECT timeInterval_Month AS Month, SUM(#{@cost_column}) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC'")
    end

    it 'should fail when non-existing dataset is given in the query' do
      query = "SELECT timeInterval_Month as Month, #{@cost_column} AS #{@cost_column} FROM AWS_CUR_NON_EXSISTENT"
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 33333, 1000, true, false, 'true' )
      expect(error).to eq('Table AWS_CUR_NON_EXSISTENT is undefined')
    end

    it 'should fail when time range is greater than 3 months for DAY granularity' do
      query = "SELECT timeInterval_Day as Day, #{@cost_column} AS #{@cost_column} FROM AWS_CUR"
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'DAILY', (Date.today-100).to_s , Date.today.to_s, query, 33333, 1000, true, false, 'true' )
      expect(error).to eq('Unsupported Time Range, Maximum allowed for DAILY Granularity is 3 MONTHS')
    end

    it 'should fail when time range is greater than 13 months for MONTHLY granularity' do
      query = "SELECT timeInterval_Month as Month, #{@cost_column} AS #{@cost_column} FROM AWS_CUR"
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY',  "#{(Date.today-425).to_s.split("-")[0]}-#{(Date.today-425).to_s.split("-")[1]}" ,  "#{(Date.today).to_s.split("-")[0]}-#{Date.today.to_s.split("-")[1]}" , query, 33333, 1000, true, false, 'true' )
      expect(error).to eq('Unsupported Time Range, Maximum allowed for MONTHLY Granularity is 13 MONTHS')
    end

    it 'should fail Giving Proper Error Message to Decrase the LIMIT on Interactive Query as Too Many Cells are Not supported' do
      query = "SELECT timeInterval_Month AS Month, perspective.accounts AS Perspective_accounts, organization.accountname AS Organization_accountname, lineItem_ProductCode AS ProductCode, lineItem_UsageType AS UsageType, lineItem_UsageAccountId AS UsageAccountId, costcategory_cht_tower AS CostCategoryChtTower, bill_CloudHealthGeneratedAccountId AS BillChGeneratedAccId, SUM(lineItem_UnblendedCost) AS SUM_LineItemUnblendedCost, SUM(#{@cost_column}) AS SUM_edpNetUnblendedCost, \"resourcetags_user:cht_tower\" AS Resourcetags_user_cht_tower, \"resourcetags_user:cht_owner\" AS Resourcetags_user_cht_owner, \"resourcetags_user:cht_function\" AS Resourcetags_user_cht_function, \"resourcetags_user:cht_env\" AS Resourcetags_user_cht_env FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_ProductCode, lineItem_UsageType, lineItem_UsageAccountId, costcategory_cht_tower, bill_CloudHealthGeneratedAccountId, perspective.accounts, organization.accountname, \"resourcetags_user:cht_tower\", \"resourcetags_user:cht_owner\", \"resourcetags_user:cht_function\", \"resourcetags_user:cht_env\" ORDER BY timeInterval_Month DESC"
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month ,  @to_month , query, 33333, 1000, true, false, 'true' )
      expect(error).to eq('Only 300000 cells can be queried at once. With this select clause, please use limit of 21428 or less.')
    end

    it 'should fail transforming the sql when there are no aliases for the columns' do
      query = 'SELECT timeInterval_Month , SUM(LineItem_UnblendedCost) FROM AWS_CUR'
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month ,  @to_month , query, 33333, 1000, true, false, 'true' )
      expect(error).to eq("Alias is not present in select expression 'timeInterval_Month'")
    end

    xit 'should fail When SQL Aliases contain special characters other than Numbers and Underscore' do
      query = "SELECT timeInterval_Month AS Month, perspective.accounts AS \"Perspective.accounts\", organization.accountname AS \"Organization.accountname\", lineItem_ProductCode AS ProductCode, lineItem_UsageType AS UsageType, lineItem_UsageAccountId AS UsageAccountId, costcategory_cht_tower AS CostCategoryChtTower, bill_CloudHealthGeneratedAccountId AS BillChGeneratedAccId, SUM(lineItem_UnblendedCost) AS \"LineItemUnblendedCost($)\", SUM(#{@cost_column}) AS \"edpNetUnblendedCost($)\", \"resourcetags_user:cht_tower\" AS \"Resourcetags_user:cht_tower\", \"resourcetags_user:cht_owner\" AS \"Resourcetags_user:cht_owner\", \"resourcetags_user:cht_function\" AS \"Resourcetags_user:cht_function\", \"resourcetags_user:cht_env\" AS \"Resourcetags_user:cht_env\" FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_ProductCode, lineItem_UsageType, lineItem_UsageAccountId, costcategory_cht_tower, bill_CloudHealthGeneratedAccountId, perspective.accounts, organization.accountname, \"resourcetags_user:cht_tower\", \"resourcetags_user:cht_owner\", \"resourcetags_user:cht_function\", \"resourcetags_user:cht_env\" ORDER BY TimeInterval_Month DESC"
      error = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month ,  @to_month , query, 3333, 1000, true, false, 'true' )
      expect(error).to eq('SQL Aliases should start with alphabets and can only contain alphabets, numeric values and _')
    end

    it 'should run a complex sql with more than 3 datasets joined in interactive api', joins: true do
      query = " with cxtemp_aws_cur as ( select LineItem_UnblendedCost as Cost, TimeInterval_Day as Day, perspective.\"Team\" AS Perspective_Team, 'aws' as CloudType FROM AWS_CUR ), cxtemp_azure as ( select CostInBillingCurrency as Cost, TimeInterval_day as Day, perspective.\"Team\" AS Perspective_Team, 'azure' as CloudType FROM AZURE_COST_USAGE ), cxtemp_gcp as ( SELECT timeInterval_Day AS Day, Cost AS Cost, 'gcp' as CloudType FROM GCP_BILLING_EXPORT WHERE (Project_Id IS NOT NULL) AND (Cost_Type LIKE '%regular%') ), cxtemp_aws_azure as ( select SUM(cxtemp_aws_cur.Cost) as AwsCost, SUM(cxtemp_azure.Cost) as AzureCost, cxtemp_azure.Day as Day, cxtemp_aws_cur.Perspective_Team as P_Team FROM cxtemp_aws_cur LEFT JOIN cxtemp_azure ON cxtemp_azure.Day = cxtemp_aws_cur.Day WHERE cxtemp_aws_cur.Perspective_Team = cxtemp_azure.Perspective_Team GROUP BY cxtemp_azure.Day, cxtemp_aws_cur.Perspective_Team ) select cxtemp_aws_azure.Day as Day, cxtemp_aws_azure.P_Team as Perspective_Team, ROUND(SUM(cxtemp_gcp.Cost), 2) as GcpCost, ROUND(SUM(cxtemp_aws_azure.Awscost), 2) as AwsCost, ROUND(SUM(cxtemp_aws_azure.AzureCost), 2) as AzureCost FROM cxtemp_aws_azure LEFT JOIN cxtemp_gcp ON cxtemp_gcp.Day = cxtemp_aws_azure.Day GROUP BY cxtemp_aws_azure.Day, cxtemp_aws_azure.P_Team "

      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'DAILY', @from_month, @to_month, query, 300, 10, true, false, 'false', 90)
      expect(failures).to be_empty
    end


    xit 'should fail when timeRange is passed to do non-time bound dataset' do
      puts "skipped"
    end

    xit 'should fail when timeRange is not passed to time bounded dataset' do
      puts "skipped"
    end

    xit 'should return error message with maximum limit allowed when limit -1 is passed into either time bounded or non-time bounded dataset' do
      puts "skipped"
    end
  end
end
