require 'spec_helper'

module MdrTest
  describe 'Mdr previewFlexReport' do
    include_context 'request metadata'

    before(:all) do
      start_time_day = Time.new - 11540000
      @from_month = (start_time_day).strftime("%Y-%m")
      @to_month = (start_time_day+13328000).strftime("%Y-%m")
      @from_month_daily_date = start_time_day.strftime("%Y-%m-%d")
      @to_month_daily_date = (start_time_day+7689600).strftime("%Y-%m-%d")
    end

    it "CAN QUERY MONTHLY GRANULARITY NO RESOURCE TAGS AGGREGATION ON BACKLINK ON" do
      sql = "SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC"
      sql = "SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) as Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC" if ENV['TEST_ENV'].eql? 'preprod'

      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY NO RESOURCE TAGS AGGREGATION ON BACKLINK OFF" do
      sql = "SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC"
      sql = "SELECT timeInterval_Month AS Month, SUM(LineItem_unblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY timeInterval_Month ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY NO RESOURCE TAGS AGGREGATION OFF BACKLINK OFF" do
      sql = "SELECT timeInterval_Month AS Month, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY timeInterval_Month ASC"
      sql = "SELECT timeInterval_Month AS Month, LineItem_UnblendedCost AS Cost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY timeInterval_Month ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY NO RESOURCE TAGS AGGREGATION OFF BACKLINK ON" do
      sql = "SELECT timeInterval_Month AS Month, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY timeInterval_Month ASC"
      sql = "SELECT timeInterval_Month AS Month, LineItem_UnblendedCost AS Cost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY timeInterval_Month ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK ON" do
      sql = "SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC"
      sql = "SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK OFF" do
      sql="SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC"
      sql="SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK OFF" do
      sql="SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC"
      sql="SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK ON" do
      sql = "SELECT timeInterval_Month AS Month, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR ORDER BY \"resourceTags_user:cht_owner\" ASC"
      sql = "SELECT timeInterval_Month AS Month, LineItem_UnblendedCost AS Cost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR ORDER BY \"resourceTags_user:cht_owner\" ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK ON WITH FILTER RESOURCE TAG" do
      sql = "SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC OFFSET 5"
      sql = "SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC OFFSET 5" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK OFF WITH FILTER RESOURCE TAG" do
      sql = "SELECT timeInterval_Month AS Month, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC"
      sql = "SELECT timeInterval_Month AS Month, SUM(LineItem_unblendedCost) AS SUM_lineItem_UnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Month, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK OFF WITH FILTER RESOURCE TAG" do
      sql="SELECT timeInterval_Month AS Month, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY bill_PayerAccountId ASC"
      sql="SELECT timeInterval_Month AS Month, LineItem_UnblendedCost AS LineItem_UnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY bill_PayerAccountId ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY MONTHLY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK ON WITH RESOURCE TAG FILTER" do
      sql="SELECT timeInterval_Month AS Month, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY bill_PayerAccountId ASC"
      sql="SELECT timeInterval_Month AS Month, LineItem_UnblendedCost AS LineItem_UnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY bill_PayerAccountId ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'MONTHLY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month,
              'to': @to_month
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY NO RESOURCE TAGS AGGREGATION ON BACKLINK ON" do
      sql="SELECT timeInterval_Day AS Day, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY lineItem_Operation ASC"
      sql="SELECT timeInterval_Day AS Day, SUM(LineItem_UnblendedCost) AS SUM_LineItem_UnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS bill_CloudHealthGeneratedAccountId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY lineItem_Operation ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY NO RESOURCE TAGS AGGREGATION ON BACKLINK OFF" do
      sql="SELECT timeInterval_Day AS Day, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY lineItem_UsageType ASC OFFSET 5"
      sql="SELECT timeInterval_Day AS Day, SUM(LineItem_UnblendedCost) AS SUM_LineItem_UnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName ORDER BY lineItem_UsageType ASC OFFSET 5" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY NO RESOURCE TAGS AGGREGATION OFF BACKLINK OFF" do
      sql="SELECT timeInterval_Day AS Day, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY 4 ASC"
      sql="SELECT timeInterval_Day AS Day, LineItem_unblendedCost AS LineItem_unblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY 4 ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY NO RESOURCE TAGS AGGREGATION OFF BACKLINK ON" do
      sql="SELECT timeInterval_Day AS Day, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY 8 ASC"
      sql="SELECT timeInterval_Day AS Day, LineItem_UnblendedCost AS LineItem_UnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName FROM AWS_CUR ORDER BY 8 ASC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK ON" do
      sql="SELECT timeInterval_Day AS Day, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" DESC"
      sql="SELECT timeInterval_Day AS Day, SUM(LineItem_UnblendedCost) AS SUM_LineItem_UnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" DESC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK OFF" do
      sql="SELECT timeInterval_Day AS Day, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" DESC"
      sql="SELECT timeInterval_Day AS Day, SUM(LineItem_UnblendedCost) AS SUM_LineItem_UnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY \"resourceTags_user:cht_owner\" DESC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK OFF" do
      sql="SELECT timeInterval_Day AS Day, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR ORDER BY \"resourceTags_user:cht_owner\""
      sql="SELECT timeInterval_Day AS Day, LineItem_UnblendedCost AS LineItem_UnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR ORDER BY \"resourceTags_user:cht_owner\"" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK ON" do
      sql="SELECT timeInterval_Day AS Day, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR ORDER BY \"resourceTags_user:cht_owner\""
      sql="SELECT timeInterval_Day AS Day, LineItem_UnblendedCost AS Cost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR ORDER BY \"resourceTags_user:cht_owner\"" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK ON WITH FILTER RESOURCE TAG" do
      sql="SELECT timeInterval_Day AS Day, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY product_ProductName DESC"
      sql="SELECT timeInterval_Day AS Day, SUM(LineItem_UnblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\" ORDER BY product_ProductName DESC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION ON BACKLINK OFF WITH FILTER RESOURCE TAG" do
      sql="SELECT timeInterval_Day AS Day, SUM(edp_NetUnblendedCost) AS SUM_edp_NetUnblendedCost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\""
      sql="SELECT timeInterval_Day AS Day, SUM(LineItem_unblendedCost) AS Cost, SUM(lineItem_UsageAmount) AS SUM_lineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') GROUP BY timeInterval_Day, bill_CloudHealthGeneratedAccountId, bill_PayerAccountId, lineItem_Operation, lineItem_UsageAccountId, lineItem_UsageType, product_ProductName, \"resourceTags_user:cht_owner\"" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK OFF WITH FILTER RESOURCE TAG" do
      sql = "SELECT timeInterval_Day AS Day, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY timeInterval_Day DESC"
      sql = "SELECT timeInterval_Day AS Day, LineItem_UnblendedCost AS Cost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY timeInterval_Day DESC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    it "CAN QUERY DAILY GRANULARITY WITH RESOURCE TAGS AGGREGATION OFF BACKLINK ON WITH FILTER RESOURCE TAG" do
      sql = "SELECT timeInterval_Day AS Day, edp_NetUnblendedCost AS Edp_NetUnblendedCost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY timeInterval_Day DESC"
      sql = "SELECT timeInterval_Day AS Day, LineItem_UnblendedCost AS Cost, lineItem_UsageAmount AS LineItem_UsageAmount, bill_CloudHealthGeneratedAccountId AS Bill_CH_GeneratedAccId, bill_PayerAccountId AS Bill_PayerAccountId, lineItem_Operation AS LineItem_Operation, lineItem_UsageAccountId AS LineItem_UsageAccountId, lineItem_UsageType AS LineItem_UsageType, product_ProductName AS Product_ProductName, \"resourceTags_user:cht_owner\" AS \"ResourceTags_user:cht_owner\" FROM AWS_CUR WHERE (\"resourceTags_user:cht_owner\" LIKE '%satyanarayaj%') ORDER BY timeInterval_Day DESC" if ENV['TEST_ENV'].eql? 'preprod'
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': true,
            'sqlStatement': sql,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:previewFlexReport, service: service)
      end
    end

    # TODO: Preview Runs for long time for thus query handle error: undefined method `code' for nil:NilClass
    xit "Can Preview Temp Table Join Query SuccessFully", preview_api_joins:true do
      sql = " with cxtemp_aws_cur as ( select LineItem_UnblendedCost as Cost, TimeInterval_Day as Day, perspective.\"Team\" AS Perspective_Team, 'aws' as CloudType FROM AWS_CUR ), cxtemp_azure as ( select CostInBillingCurrency as Cost, TimeInterval_day as Day, perspective.\"Team\" AS Perspective_Team, 'azure' as CloudType FROM AZURE_COST_USAGE ), cxtemp_gcp as ( SELECT timeInterval_Day AS Day, Cost AS Cost, 'gcp' as CloudType FROM GCP_BILLING_EXPORT WHERE (Project_Id IS NOT NULL) AND (Cost_Type LIKE '%regular%') ), cxtemp_aws_azure as ( select SUM(cxtemp_aws_cur.Cost) as AwsCost, SUM(cxtemp_azure.Cost) as AzureCost, cxtemp_azure.Day as Day, cxtemp_aws_cur.Perspective_Team as P_Team FROM cxtemp_aws_cur LEFT JOIN cxtemp_azure ON cxtemp_azure.Day = cxtemp_aws_cur.Day WHERE cxtemp_aws_cur.Perspective_Team = cxtemp_azure.Perspective_Team GROUP BY cxtemp_azure.Day, cxtemp_aws_cur.Perspective_Team ) select cxtemp_aws_azure.Day as Day, cxtemp_aws_azure.P_Team as Perspective_Team, ROUND(SUM(cxtemp_gcp.Cost), 2) as GcpCost, ROUND(SUM(cxtemp_aws_azure.Awscost), 2) as AwsCost, ROUND(SUM(cxtemp_aws_azure.AzureCost), 2) as AzureCost FROM cxtemp_aws_azure LEFT JOIN cxtemp_gcp ON cxtemp_gcp.Day = cxtemp_aws_azure.Day GROUP BY cxtemp_aws_azure.Day, cxtemp_aws_azure.P_Team "
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'DAILY',
            'needBackLinkingForTags': false,
            'sqlStatement': sql,
            'timeRange': {
              'last': 90
            }
          }
        }
        @retry_count = 0
        begin
          puts " PreviewFlexReport Api with retry count: #{@retry_count}"
          graphql url(service: service),
                  format_request(
                    :previewFlexReport,
                    service: service
                  ),
                  variables, headers
          expect_success_graphql
          check(:previewFlexReport, service: service)
        rescue NoMethodError => error
          @retry_count += 1
          retry if @retry_count < 3
          raise("Error Occurred While Fetching getFlexReportQueryResult with error #{error.message}") if @retry_count >= 3
        end
      end
    end

    xit 'E:no auth' do
      wrap_request do
        variables = {
          'queryInput': {
            'dataGranularity': 'dataGranularity',
            'limit': 'limit',
            'needBackLinkingForTags': 'needBackLinkingForTags',
            'sqlStatement': 'sqlStatement',
            'timeRange': {
              'from': 'from',
              'last': 'last',
              'to': 'to'
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :previewFlexReport,
                  service: service
                ),
                variables, apitest_headers
        expect_status(401)
        expect_error_graphql(
          status: 401,
          errors: [{ message: 'Invalid or missing authentication token' }]
        )
      end
    end
  end
end
