require 'spec_helper'
require 'securerandom'

include ChtApitestMdr::MdrHelpers

module MdrFlexReportTest

  describe 'FlexReports' do
    include_context 'request metadata'

    before(:all) do
      EXPECTED_JSON_METADATA = JSON.parse(COLUMN_METADATA)
      # Report Id generated during test
      @report_name = "rspec-column-metadata-validation-#{ENV['NAMESPACE']}-#{rand(999999999999999)}"
      @report_ids = []
      @dataset = 'AWS_CUR'
      # @sql_functions_sql = "WITH \"cxtemp_table\" AS (SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS UnblendedCost, SUM(Reservation_AmortizedUpfrontCostForUsage) AS RESERVATION_AMORTIZED_COST, SUM(SavingsPlan_AmortizedCostForUsage) AS SP_AMORTIZED_COST, lineItem_UsageAccountId AS LineItem_UsageAccountId, product_ProductName AS Product_ProductName, COALESCE(product_region, SavingsPlan_Region) AS Region, COUNT(Product_ProductFamily) AS ProductFamiliesPerProduct, SPLIT_PART(COALESCE(reservation_ReservationARN, savingsPlan_SavingsPlanARN), ':', 6) AS ARN, lineItem_ProductCode AS LineItem_ProductCode, COALESCE(savingsPlan_PurchaseOption, pricing_PurchaseOption) AS PurchaseOption, COALESCE(DATE(FROM_ISO8601_TIMESTAMP(Reservation_StartTime)), DATE(FROM_ISO8601_TIMESTAMP(SavingsPlan_StartTime))) AS YYYY_MM_DD_START_TIME FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_UsageAccountId, product_ProductName, product_region, reservation_ReservationARN, savingsPlan_SavingsPlanARN, savingsPlan_Region, savingsPlan_PurchaseOption, lineItem_ProductCode, pricing_PurchaseOption, SavingsPlan_StartTime, Reservation_StartTime, Product_ProductFamily) SELECT ProductFamiliesPerProduct AS ProductFamilyCount, Month AS Month, Product_ProductName AS Product, SUM( CASE WHEN PurchaseOption LIKE '%OnDemand%' THEN cast(UnblendedCost as DECIMAL(18,3)) END) AS ONDEMANDCOST, SUM( CASE WHEN PurchaseOption LIKE '%No%' THEN cast(RESERVATION_AMORTIZED_COST as DECIMAL(18,3)) END) AS RES_COST, SUM( CASE WHEN PurchaseOption LIKE '%Partial%' THEN cast(SP_AMORTIZED_COST as DECIMAL(18,3)) END) AS SP_COST ,LineItem_UsageAccountId as UsageAccount, PurchaseOption as PurchaseOption  FROM cxtemp_table GROUP BY ProductFamiliesPerProduct, Month, Product_ProductName, PurchaseOption, LineItem_UsageAccountId ORDER BY Month DESC"
      # @sql_functions_sql = "WITH \"cxtemp_table\" AS (SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS UnblendedCost, SUM(Reservation_AmortizedUpfrontCostForUsage) AS RESERVATION_AMORTIZED_COST, SUM(SavingsPlan_AmortizedCostForUsage) AS SP_AMORTIZED_COST, lineItem_UsageAccountId AS LineItem_UsageAccountId, product_ProductName AS Product_ProductName, COALESCE(product_region, SavingsPlan_Region) AS Region, COUNT(Product_ProductFamily) AS ProductFamiliesPerProduct, lineItem_ProductCode AS LineItem_ProductCode, COALESCE(savingsPlan_PurchaseOption, pricing_PurchaseOption) AS PurchaseOption, COALESCE(DATE(FROM_ISO8601_TIMESTAMP(Reservation_StartTime)), DATE(FROM_ISO8601_TIMESTAMP(SavingsPlan_StartTime))) AS YYYY_MM_DD_START_TIME FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_UsageAccountId, product_ProductName, product_region, savingsPlan_Region, savingsPlan_PurchaseOption, lineItem_ProductCode, pricing_PurchaseOption, SavingsPlan_StartTime, Reservation_StartTime, Product_ProductFamily) SELECT ProductFamiliesPerProduct AS ProductFamilyCount, Month AS Month, Product_ProductName AS Product, SUM( CASE WHEN PurchaseOption LIKE '%OnDemand%' THEN cast(UnblendedCost as DECIMAL(18,3)) END) AS ONDEMANDCOST, SUM( CASE WHEN PurchaseOption LIKE '%No%' THEN cast(RESERVATION_AMORTIZED_COST as DECIMAL(18,3)) END) AS RES_COST, SUM( CASE WHEN PurchaseOption LIKE '%Partial%' THEN cast(SP_AMORTIZED_COST as DECIMAL(18,3)) END) AS SP_COST ,LineItem_UsageAccountId as UsageAccount, PurchaseOption as PurchaseOption  FROM cxtemp_table GROUP BY ProductFamiliesPerProduct, Month, Product_ProductName, PurchaseOption, LineItem_UsageAccountId ORDER BY Month DESC" if ENV['TEST_ENV'].eql? 'preprod'
      @sql_functions_sql = "WITH \"cxtemp_table\" AS (SELECT timeInterval_Month AS Month, SUM(LineItem_UnblendedCost) AS UnblendedCost, SUM(Reservation_AmortizedUpfrontCostForUsage) AS RESERVATION_AMORTIZED_COST, SUM(SavingsPlan_AmortizedCostForUsage) AS SP_AMORTIZED_COST, lineItem_UsageAccountId AS LineItem_UsageAccountId, product_ProductName AS Product_ProductName, COALESCE(product_region, SavingsPlan_Region) AS Region, COUNT(Product_ProductFamily) AS ProductFamiliesPerProduct, lineItem_ProductCode AS LineItem_ProductCode, COALESCE(savingsPlan_PurchaseOption, pricing_PurchaseOption) AS PurchaseOption, COALESCE(DATE(FROM_ISO8601_TIMESTAMP(Reservation_StartTime)), DATE(FROM_ISO8601_TIMESTAMP(SavingsPlan_StartTime))) AS YYYY_MM_DD_START_TIME FROM AWS_CUR GROUP BY timeInterval_Month, lineItem_UsageAccountId, product_ProductName, product_region, savingsPlan_Region, savingsPlan_PurchaseOption, lineItem_ProductCode, pricing_PurchaseOption, SavingsPlan_StartTime, Reservation_StartTime, Product_ProductFamily) SELECT ProductFamiliesPerProduct AS ProductFamilyCount, Month AS Month, Product_ProductName AS Product, SUM( CASE WHEN PurchaseOption LIKE '%OnDemand%' THEN cast(UnblendedCost as DECIMAL(18,3)) END) AS ONDEMANDCOST, SUM( CASE WHEN PurchaseOption LIKE '%No%' THEN cast(RESERVATION_AMORTIZED_COST as DECIMAL(18,3)) END) AS RES_COST, SUM( CASE WHEN PurchaseOption LIKE '%Partial%' THEN cast(SP_AMORTIZED_COST as DECIMAL(18,3)) END) AS SP_COST ,LineItem_UsageAccountId as UsageAccount, PurchaseOption as PurchaseOption  FROM cxtemp_table GROUP BY ProductFamiliesPerProduct, Month, Product_ProductName, PurchaseOption, LineItem_UsageAccountId ORDER BY Month DESC" if ENV['TEST_ENV'].eql? 'preprod'
    end

    it 'Create Report With any Granularity and relative time range', :create do
      wrap_request do
        variables = {
          'input': {
            'name': @report_name,
            'notification': {
              'sendUserEmail': false,
              'otherEmailIds': []
            },
            'query': {
              'dataGranularity': 'MONTHLY',
              'limit': -1,
              'needBackLinkingForTags': false,
              'sqlStatement': @sql_functions_sql,
              'timeRange': {
                'last': 11
              }
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        @report_ids << json_body[:data][:createFlexReport][:id]
        expect(%w[QUEUED RUNNING]).to include(json_body[:data][:createFlexReport][:result][:status])
        expect(json_body[:data][:createFlexReport][:query][:dataGranularity]).to eql 'MONTHLY'
        # check(:createFlexReport, service: service)
      end
    end

    it "check the Created Report Status is NOT FAILED" do
      report_id = @report_ids.first
      wrap_request do
        variables = {}
        graphql url(service: service),
                format_request(
                  :node,
                  service: service,
                  crn: report_id,
                  type: 'FlexReport',
                  fields: 'name archived metadata { columnInfos { columnUnit name namespace type dataType } columnCardinalityInfo {columnName cardinality} }  result { contentType contents { preSignedUrl } message reportUpdatedOn status } query { sqlStatement dataGranularity timeRange { last } }'
                ),
                variables, headers
        expect_success_graphql
        check(:node, service: service)
        @@metadata = json_body[:data][:node][:metadata]
        expect(json_body[:data][:node][:id]).to eql report_id
        expect(json_body[:data][:node][:name]).to eq  @report_name
        expect(json_body[:data][:node][:result][:status]).not_to eql 'FAILED'
      end
    end

    it 'validates metadata is returned as expected from the BE API' do
      expect(EXPECTED_JSON_METADATA.to_json).to eql(@@metadata.to_json)
    end



    it "can DELETE an existing report" do
      report_id = @report_ids.first
      wrap_request do
        variables = {
          'id': report_id
        }
        graphql url(service: service),
                format_request(
                  :delete,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:delete, service: service)
        expect(json_body[:data][:delete]).to eql true
      end
    end

    xit 'E:no auth' do
      wrap_request do
        variables = {
          'archived': false,
          'dataSource': @data_source
        }
        graphql url(service: service),
                format_request(
                  :flexReports,
                  service: service
                ),
                variables, apitest_headers
        expect_status(403)
        expect_error_graphql(
          status: 403,
          errors: [{ message: 'AUTHN_403' }]
        )
      end
    end
  end
end
